using System.ComponentModel.DataAnnotations;

using CommunityToolkit.Mvvm.ComponentModel;

namespace NSeguin.Dev.Web;

public partial class ContactFormVM : ObservableValidator
{
    [ObservableProperty]
    [Required(ErrorMessage = "Please enter your name.")]
    private string? _name;

    [ObservableProperty]
    [EmailAddress(ErrorMessage = "The email address is not valid.")]
    [Required(ErrorMessage = "Please enter your email address.")]
    private string? _email;

    [ObservableProperty]
    [Phone(ErrorMessage = "The phone number is not valid.")]
    private string? _phoneNumber;

    [ObservableProperty]
    [Required(ErrorMessage = "Please enter a message.")]
    private string? _message;

    [ObservableProperty]
    private bool _isSending;

    [ObservableProperty]
    private bool _isSent;

    [ObservableProperty]
    private bool _isHidden = true;

    public ContactFormVM()
    {
        Sending += SendingEventHandler;
        Sent += SentEventHandler;
    }

    public event EventHandler<ContactFormSendingEventArgs>? Sending;
    public event EventHandler<ContactFormSentEventArgs>? Sent;

    public void Send()
    {
        if (HasErrors || IsSending || IsSent)
        {
            return;
        }

        OnSending(new ContactFormSendingEventArgs(this));
    }

    public void NotifySent(bool isSuccessful)
    {
        OnSent(new ContactFormSentEventArgs(isSuccessful));
    }

    public void Reset()
    {
        Name = null;
        Email = null;
        PhoneNumber = null;
        Message = null;
        IsSending = false;
        IsSent = false;
    }

    public void Hide()
    {
        IsHidden = true;
    }

    public void Show()
    {
        IsHidden = false;
    }

    public void Toggle()
    {
        IsHidden = !IsHidden;
    }

    private void OnSending(ContactFormSendingEventArgs e)
    {
        Sending?.Invoke(this, e);
    }

    private void OnSent(ContactFormSentEventArgs e)
    {
        Sent?.Invoke(this, e);
    }

    private void SendingEventHandler(object? sender, ContactFormSendingEventArgs e)
    {
        IsSent = false;
        IsSending = true;
    }

    private void SentEventHandler(object? sender, ContactFormSentEventArgs e)
    {
        IsSending = false;
        IsSent = e.IsSuccessful;
    }
}
