using Azure.Identity;

using Microsoft.AspNetCore.Authentication.OpenIdConnect;
using Microsoft.Identity.Web;
using Microsoft.Identity.Web.UI;
using Microsoft.IdentityModel.Protocols.OpenIdConnect;

using NSeguin.Dev.Web;
using NSeguin.Dev.Web.Client.Pages;
using NSeguin.Dev.Web.Components;
using NSeguin.Dev.Web.Extensions;
using NSeguin.Dev.Web.Services;

WebApplicationBuilder builder = WebApplication.CreateBuilder(args);
builder.Logging.AddConfiguration(builder.Configuration.GetSection("Logging"));
builder.Logging.AddConsole();
builder.Services.Configure<CookiePolicyOptions>(
    options =>
    {
        // This lambda determines whether user consent for non-essential cookies is needed for a given request.
        options.CheckConsentNeeded = context => true;
        options.MinimumSameSitePolicy = SameSiteMode.Unspecified;
        // Handling SameSite cookie according to https://docs.microsoft.com/en-us/aspnet/core/security/samesite?view=aspnetcore-3.1
        options.HandleSameSiteCookieCompatibility();
    });

builder.Services.AddControllersWithViews().AddMicrosoftIdentityUI();
builder.Services.AddRazorPages();
builder.Services.AddRazorComponents()
    .AddInteractiveServerComponents()
    .AddInteractiveWebAssemblyComponents();

if (builder.Environment.IsStaging() || builder.Environment.IsProduction())
{
    builder.Configuration.AddAzureAppConfiguration(
        a =>
        {
            string? connectionString
                = Environment.GetEnvironmentVariable("AZURE_APP_CONFIGURATION_CONNECTION_STRING");

            if (string.IsNullOrWhiteSpace(connectionString))
            {
                throw new InvalidOperationException(
                    "The environment variable AZURE_APP_CONFIGURATION_CONNECTION_STRING is not set.");
            }

            a.Connect(connectionString)
                .ConfigureKeyVault(kv => kv.SetCredential(new DefaultAzureCredential()));
        });
}
else
{
    builder.Configuration.AddUserSecrets<Program>();
}

builder.Services.AddAuthentication(OpenIdConnectDefaults.AuthenticationScheme)
    .AddMicrosoftIdentityWebApp(
        builder.Configuration.GetSection("AzureAdB2C"),
        subscribeToOpenIdConnectMiddlewareDiagnosticsEvents: true);

builder.Services.Configure<OpenIdConnectOptions>(
    OpenIdConnectDefaults.AuthenticationScheme,
    options =>
    {
        options.ResponseType = OpenIdConnectResponseType.Code;
        options.Scope.Add(options.ClientId);
    });

builder.Services.AddTransient<IContactService, ContactService>();
builder.Services.AddOptions<MailSettings>()
    .BindConfiguration("Mail")
    .ValidateDataAnnotations()
    .ValidateOnStart();

builder.Services.AddOptions<SmtpSettings>()
    .BindConfiguration("Mail:Smtp")
    .ValidateDataAnnotations()
    .ValidateOnStart();

builder.Services.AddHttpLogging(
    o =>
    {
        o.CombineLogs = true;
    });

builder.Services.AddNSeguinDevWebServer();

WebApplication app = builder.Build();

// Configure the HTTP request pipeline.
if (app.Environment.IsDevelopment())
{
    app.UseWebAssemblyDebugging();
}
else
{
    app.UseExceptionHandler("/Error", true);
    // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
    app.UseHsts();
}

app.UseHttpLogging();
app.UseHttpsRedirection();
app.UseBlazorFrameworkFiles();
app.UseStaticFiles(new StaticFileOptions {ServeUnknownFileTypes = true});
app.UseRouting();
app.UseAuthentication();
app.UseAuthorization();
app.UseAntiforgery();
app.MapRazorComponents<App>()
    .AddAdditionalAssemblies(typeof(Counter).Assembly)
    .AddInteractiveServerRenderMode()
    .AddInteractiveWebAssemblyRenderMode();

app.MapControllers();
app.MapFallback(
    ctx =>
    {
        ctx.Response.Redirect($"/NotFound?path={ctx.Request.Path}");
        return Task.CompletedTask;
    });

app.Run();
